/*
 *  MyXMLApplet.java
 *
 *  Created on September 27, 2002, 4:00 PM
 *  updated by JMH on October 05, 2002 at 09:17 AM
 *
 *  jar command used:
 *  jar cvf umlgui.jar edu/uml/gui/UmlGuiException.class edu/uml/gui/UmlGuiException.java
 *
 *  To compile this .java file on the WebLabPCs, use:
 *     javac -classpath .:/opt/JRun/lib/ext/xerces.jar MyXMLApplet.java
 */

import java.awt.* ;             // required for Swing, too
import javax.swing.* ;          // the standard Swing package

import javax.xml.parsers.* ;    // for DocumentBuilderFactory and DocumentBuilder
import org.w3c.dom.* ;          // for Document and the Document Object Model
import org.xml.sax.* ;          // for SAXException
import java.io.* ;              // for IOException
import java.net.* ;             // for URL

import edu.uml.gui.* ;          // the UML GUI package


/** This program demonstrates reading an XML document from a server, 
 *  manipulating it, and writing it back to the server.
 *  @author  Jesse Heines, <a href="mailto:heines@cs.uml.edu">heines@cs.uml.edu</a>
 *  @version 1.9, updated by JMH on October 04, 2002 at 08:18 PM
 */
public class MyXMLApplet extends javax.swing.JApplet
{
  /** test case number to run */
  int intCaseNo = 5 ;
  
  /** applet content pane */
  Container c ;
  
  /** the path on the server to the XML file to open */
  private String strXMLInputFilePath ; 
  // note that this path must always be on the server from which the applet was
  // loaded, therefore it is best to prefix the file path with getCodeBase()
  
  /** the XML Document object */
  private Document doc ;
  
  /** text area to hold output */
  JTextArea jtxtOut ;
  

  /** Creates a new instance of TestArchive */
  public MyXMLApplet()
  {
  }
  

  /** the standard JApplet init method, called once when the applet starts */
  public void init()
  {
    // Notes:
    // (1)  an applet can only access files on the server from which it was 
    //        loaded
    // (2)  recall that getCodeBase() returns the path on the server from 
    //        which this applet was loaded
    // (3)  to get only the server from which this applet was loaded so that 
    //        you can construct a different path on that server, use 
    //        getCodeBase().getHost()
    // (4)  the base of the path where your XML files will be stored is always
    //        /students/ + the user name you specify, therefore to read a file
    //        that you previously stored, use "/students/username/..."
    switch ( intCaseNo )
    {
      case 1 : 
        strXMLInputFilePath = getCodeBase() + "j2.xml" ;
        break ;
      case 2 : 
        strXMLInputFilePath = "http://" + getCodeBase().getHost() + 
                         "/~heines/91.461/91.461-2002-03f/course.xml" ;
        break ;
      case 3 : 
        strXMLInputFilePath = "http://philip/~heines/91.513/91.513-2002-03f/course.xml" ;
        break ;
      case 4 : 
        strXMLInputFilePath = getCodeBase() + "j3.xml" ;
        break ;
      case 5 : 
        strXMLInputFilePath = "http://" + getCodeBase().getHost() + 
          "/students/heines/uploads/linux/j3.xml" ;
        break ;
      case 6 : 
        strXMLInputFilePath = "http://" + getCodeBase().getHost() + 
                         "/students/heines/j3a.xml" ;
        break ;
    }
    // System.out.println( strXMLInputFilePath ) ;

    // get a reference to the applet content pane
    c = getContentPane() ;

    // put a scrollable text area on the screen
    Font font16 = new Font( "Courier New", Font.BOLD, 16 ) ;
    jtxtOut = new JTextArea() ;
    jtxtOut.setFont( font16 ) ;
    JScrollPane jspnOut = new JScrollPane( jtxtOut ) ;
    c.add( jspnOut, BorderLayout.CENTER ) ;
    
    // initialize document
    try {
      doc = UmlGui.openXMLDocument( strXMLInputFilePath ) ;
      // if ( doc == null )
      //   System.out.println( "null" ) ;
      // else
      //   System.out.println( doc.getDocumentElement() ) ;
    } catch ( UmlGuiException uge ) {
      System.out.println( uge.toString() ) ;
    }

  } // init()
  

  /** the standard JApplet start method, called each time the page is visited */
  public void start()
  {
    String strText = "" ;

    // show information about this applet
    strText += "Applet Loaded at:  " + (new java.util.Date()).toString() ;
    strText += "\nTest Case Number:  " + intCaseNo ;
    strText += "\nApplet Directory:  " + getCodeBase().toString() ;
    strText += "\n  XML Input File:  " + strXMLInputFilePath ;
    
    if ( doc == null ) 
    {
      strText += "\n\ndoc is null" ;
      jtxtOut.setText( strText ) ;
      return ;
    }
    
    // show some of the data from the XML file
    strText += "\n" + showSomeXMLData() ;
    // manipulate the data
    modifyXMLData() ;
    // show some of the data again
    strText += "\n" + showSomeXMLData() ;

    // store the XML file back on the server
    String strReturn = null ;

    try {
      switch ( intCaseNo )
      {
        case 1 :
          strReturn = UmlGui.storeXMLDocument( this, doc, "j2.xml", 
                "uml/gui/one" ) ;
          break ;
        case 2 :
          strReturn = UmlGui.storeXMLDocument( this, doc, "course.xml", 
                "uml/gui/91.461" ) ;
          break ;
        case 3 :
          strReturn = UmlGui.storeXMLDocument( this, doc, "course.xml", 
                "uml/gui/91.513" ) ;
          break ;
        case 4 :
        case 5 :
          strReturn = UmlGui.storeXMLDocument( this, doc, "j3.xml", 
            "uploads/linux/" ) ;
          break ;
        case 6 :
          strReturn = UmlGui.storeXMLDocument( this, doc, "j3a.xml" ) ;
          break ;
      }
    } catch ( UmlGuiException uge ) {
      System.out.println( uge.toString() ) ;
    }

    // test return value
    if ( strReturn != null ) 
    {
      strReturn = strReturn.trim() ;
      if ( strReturn.startsWith( "http:" ) )
        strText += "\n\nYour document was stored at URL:\n   " + strReturn ;
      else
        strText += "\nError: " + strReturn ;
    }
    
    // display all results in the text area
    jtxtOut.setText( strText ) ;
  }
  
  
  /** this method returns some of the XML data in a printable string just to
   *  show what it looks like
   */
  private String showSomeXMLData()
  {
    // initialize the string to return
    String strText = "" ;
    
    // get the root element
    Element root = doc.getDocumentElement() ;
    
    // initialize a list of nodes
    NodeList nodes = null ;

    // populate the list of nodes appropriately for the different cases
    switch ( intCaseNo )
    {
      case 1 : 
      case 4 : 
      case 5 : 
      case 6 : 
        nodes = root.getElementsByTagName( "name" ) ;
        break ;
      case 2 :
      case 3 :
        nodes = root.getElementsByTagName( "student" ) ;
        break ;
    }

    // generate output from the list of node appropriate to the different cases
    if ( nodes != null )
      for ( int k = 0 ; k < nodes.getLength() && k < 5 ; k++ )
      {
        strText += "\nNode #" + ( k+1 < 10 ? " " : "" ) + (k+1) + ":  " ;
        switch ( intCaseNo )
        {
          case 1 :
          case 4 :
          case 5 :
          case 6 :
            strText += ((Element) nodes.item( k )).getFirstChild().getNodeValue() ; 
            break ;
          case 2 :
          case 3 :
            strText += ((Element) nodes.item( k )).getAttribute( "name" ) ;
            break ;
        }
      }
    
    // return the generated output
    return strText ;
  }


  /** this method modifies the XML data a bit just to show what might be done
   */
  private void modifyXMLData()
  {
    // get the root element
    Element root = doc.getDocumentElement() ;
    
    // initialize a list of nodes
    NodeList nodes = null ;

    // populate the list of nodes appropriately for the different cases
    switch ( intCaseNo )
    {
      case 1 : 
      case 4 : 
      case 5 : 
      case 6 :
        nodes = root.getElementsByTagName( "name" ) ;
        break ;
      case 2 :
      case 3 :
        nodes = root.getElementsByTagName( "student" ) ;
        break ;
    }

    // reverse the order of the nodes
    if ( nodes != null )
    {
      // create a new element to serve as the root which will hold the 
      // manipulated data
      Element elemNewRoot = doc.createElement( root.getTagName() ) ;
    
      // append the existing nodes in reverse order
      for ( int k = nodes.getLength() - 1 ; k > -1 ; k-- )
      {
        elemNewRoot.appendChild( nodes.item( k ) ) ;
      }
      
      // replace the old document root with the new one
      doc.replaceChild( elemNewRoot, root ) ;
    }
  }



}
