/*
 *  UmlGui.java
 *  by Jesse M. Heines, heines@cs.uml.edu
 *
 *  Created on September 27, 2002, 7:29 PM
 *
 *  Note: To use this class, files xercesImpl.jar and xerces.jar must be 
 *  installed in the lib\ext directory of the JRE directory that is being 
 *  used by Internet Explorer.  On my systems, this is 
 *  C:\Program Files\Java\j2re1.4.0_02\lib\ext.
 *
 *  To compile this class, xerces.jar and xercesImpl.jar must be in the 
 *  classpath.  The easiest way to do this is to put these files in the 
 *  jre\lib\ext directory of the Java Development Kit home.  On my systems,
 *  this is C:\Program Files\j2sdk1.4.0_02\jre\lib\ext.
 *
 *  Files xerces.jar and xercesImpl.jar are downloadable from 
 *  http://xml.apache.org/dist/xerces-j/, where they are contained within 
 *  file Xerces-J-bin.2.1.0.zip.  That ZIP file includes three .jar files 
 *  as well as all of the API documentation.
 *
 *  To compile this .java file on the WebLabPCs, cd to its directory and
 *  enter:
 *     javac -classpath ../../..:/opt/JRun/lib/ext/xerces.jar UmlGui.java
 */
package edu.uml.gui ;

import java.awt.* ;             // required for Swing, too
import javax.swing.* ;          // the standard Swing package, for getCodeBase
import java.applet.* ;          // for Applet

import javax.xml.parsers.* ;    // for DocumentBuilderFactory and DocumentBuilder
import org.w3c.dom.* ;          // for Document and the Document Object Model
import org.xml.sax.* ;          // for SAXException
import java.io.* ;              // for IOException
import java.net.* ;             // for URL

import org.apache.xml.serialize.* ;   // for OutputFormat


/** This class contains all the methods available in the edu.uml.gui package.
 *
 *  @author  Jesse M. Heines, <a href="mailto:heines@cs.uml.edu">heines@cs.uml.edu</a>
 *  @version 1.0, updated by JMH on October 05, 2002 at 11:15 AM
 */
public class UmlGui {
  
  /** location of umlguiStoreXML.jsp */
  private static String strServletURI = "/~heines/xml/umlguiStoreXML9.jsp" ;
  /** location of umlguiStoreXML servlet */
  // private static String strServletURI = "/~heines/servlet/umlguiStoreXML6" ;


  /** Creates a new instance of UmlGui */
  public UmlGui() {
  }
  

  /** This <code>public static</code> method opens and XML file and returns the 
   *  <code>Document</code> object representing the XML document it contains.</p>
   *  <p>If the file cannot be opened or parsed, a <code>UmlGuiException</code> is 
   *  thrown. This is a checked exception derived from <code>IOException</code>, 
   *  so this method must be called from within a <code>try</code> block.
   *  @param strFilePath full path to the XML file to be opened
   *  @return <code>Document</code> object for that file
   *  @throws UmlGuiException
   */
  public static Document openXMLDocument( String strFilePath ) 
      throws UmlGuiException
  {
    // (1) get an instance of the DocumentBuilderFactory class
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance() ;
    
    // (2) use the instance of the DocumentBuilderFactory class to get 
    // an instance of the DocumentBuilder class, noting that the call 
    // to newDocumentBuilder() can throw a ParserConfigurationException
    DocumentBuilder db = null ;  // declare outside of try block 
    try {
      db = dbf.newDocumentBuilder() ;
    } catch ( ParserConfigurationException pce ) {
      String err = pce.toString() ;
      throw new UmlGuiException( 
          "\nParser Configuration Error (ParserConfigurationException) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    }

    // (3) use the instance of the DocumentBuilder class to open and parse the 
    // document, returning the Document object if successful
    try {
      return db.parse( strFilePath ) ;
    } catch ( SAXException saxe ) {
      String err = saxe.toString() ;
      throw new UmlGuiException( 
          "\nParsing Error (SAXException) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    } catch ( FileNotFoundException fnfe ) {
      String err = fnfe.toString() ;
      throw new UmlGuiException( 
          "\nFile Not Found Error ( FileNotFoundException ) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    } catch ( IOException ioe ) {
      String err = ioe.toString() ;
      throw new UmlGuiException( 
          "\nI/O Error ( IOException ) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    }
  } // openXMLDocument(...)

  
  /** This <code>private static</code> method saves an XML Document on a 
   *  specified system. It is the main version of this method that is called 
   *  by all overloaded methods with the same name.</p>
   *  <p>It works by calling the current version of <code>umlguiStoreXML.jsp</code> 
   *  (specified in <code>private static String strServletURI</code>) on the 
   *  system specified in the first parameter.  Note that it will only work if 
   *  the system specified in the first parameter is the same as that from which 
   *  the calling applet was loaded.</p>
   *  <p>The file is saved in the user area specified in the third parameter, 
   *  under the name specified in the fourth parameter, and in the subdirectory 
   *  specified in the fifth parameter.</p>
   *  <p>Note that if a file exists with the same name in the same area and the
   *  same subdirectory, it is overwritten.</p>  
   *  <p>If an error occurs, a <code>UmlGuiException</code> is thrown.  This is a 
   *  checked exception derived from <code>IOException</code>, so this method must 
   *  be called from within a <code>try</code> block.</p>
   *  @param strHost      name of host system on which to store the document
   *  @param doc          XML document to save
   *  @param strFileName  name under which to store the document
   *  @param strFilePath  subdirectory of the user area in which to store the 
   *                        document
   *  @param strUserName  name of user area in which to store the document
   *  @return a <code>String</code> indicating the full path to the XML document
   *      stored
   *  @throws UmlGuiException
   */
  private static String storeXMLDocument( String strHost, Document doc, 
        String strFileName, String strFilePath, String strUserName ) 
      throws UmlGuiException
  {
    //// clean up passed parameters
    
    // remove excess characters from user name
    strUserName = strUserName.trim() ;
    
    // user name cannot contain spaces
    if ( strUserName.indexOf( " " ) != -1 )
      throw new UmlGuiException( 
          "\nInvalid user name in storeXMLDocument:" + 
          "\n   the user name cannot contain spaces" ) ;
    
    // file name cannot contain a path (slashes)
    strFileName = strFileName.trim() ;
    if ( strFileName.indexOf( "/" ) != -1 )
      throw new UmlGuiException( 
          "\nInvalid file name in storeXMLDocument:" + 
          "\n   the file name cannot contain a path (no slashes (\"/\") " +
          "are allowed)" ) ;
    
    // file name cannot contain spaces
    if ( strFileName.indexOf( " " ) != -1 )
      throw new UmlGuiException( 
          "\nInvalid file name in storeXMLDocument:" + 
          "\n   the file name cannot contain spaces" ) ;
    
    // remove leading and trailing slashes from file path
    if ( strFilePath != null )
    {
      strFilePath = strFilePath.trim() ;
      while ( strFilePath.startsWith( "/" ) )
        strFilePath = strFilePath.substring( 1 ) ;
      while ( strFilePath.endsWith( "/" ) )
        strFilePath = strFilePath.substring( 0, strFilePath.length()-1 ) ;
    }
    
    // replace null file path with ""
    if ( strFilePath == null ) 
      strFilePath = "" ;
    
    // file path cannot contain back slashes
    if ( strFilePath != null && strFilePath.indexOf( "\\" ) != -1 )
      throw new UmlGuiException( 
          "\nInvalid Path Error in storeXMLDocument:" + 
          "\n   the file path cannot contain back slashes (\"\\\"):  " + 
              strFilePath + 
          "\n   use only forward slashes (\"/\") in the path specification" ) ;
    
    // file path cannot contain spaces
    if ( strFilePath.indexOf( " " ) != -1 )
      throw new UmlGuiException( 
          "\nInvalid file path in storeXMLDocument:" + 
          "\n   the file path cannot contain spaces" ) ;
    
    // remove excess characters from host name
    if ( strHost != null ) 
      strHost = strHost.trim() ;
    
    ////  send the XML document to the server
    ////  reference:  Hall & Brown, pp. 1083-1090
    ////  steps referred to are on pp. 1084-1085

    // Step 1
    URL urlData ;
    String strProtocol = "http" ;
    int intPort = -1 ;
    
    try {
      urlData = new URL( strProtocol, strHost, intPort, strServletURI ) ;
    } catch ( MalformedURLException murle ) {
      throw new UmlGuiException( 
          "\nMalformedURLException at Step 1 in storeXMLDocument:" + 
          "\n   " + murle.toString() ) ;
    }
    // System.out.println( "urlData = " + urlData ) ;

    // Step 2 - establish URL connection
    URLConnection urlcConn ;
    try {
      urlcConn = urlData.openConnection() ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 2 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "urlcConn = " + urlcConn ) ;
    
    // Step 3 - prevent browser from caching results
    urlcConn.setUseCaches( false ) ;
    
    // Step 4 - allow system to send as well as receive data
    urlcConn.setDoOutput( true ) ;
    
    // Step 5 - creating stream to buffer data that will be sent
    ByteArrayOutputStream baosByteStream = new ByteArrayOutputStream( 512 ) ;
    // System.out.println( "baosByteStream = " + baosByteStream ) ;
    
    // Step 6a (see p. 1075 Step 2) - attaching an output stream
    ObjectOutputStream oosOut ;
    try {
      oosOut = new ObjectOutputStream( baosByteStream ) ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 6a in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "oosOut = " + oosOut ) ;
 
    // Step 6b - make a serialized version of the XML Document object
    // reference:  www-106.ibm.com/developerworks/xml/library/x-injava2/?dwzone=xml
    OutputFormat of = new OutputFormat( doc ) ;
    of.setIndenting( true ) ;
    of.setIndent( 2 ) ;
    // of.setPreserveSpace( true ) ;
    StringWriter sw = new StringWriter() ;
      // use a StringWriter instead of serializing directly to the 
      // ObjectOutputStream because serializing directly yields a 
      // java.io.OptionalDataException when reading the data with 
      // ObjectInputStream.readObject()
    XMLSerializer xmlser = new XMLSerializer( sw, of ) ;
    try {
      xmlser.serialize( doc ) ;
    } catch ( IOException ioe ) {
       throw new UmlGuiException( 
          "\nIOException at Step 6b in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "Serialization done." ) ;
    
    // Step 7 - put the data to be sent into the output buffer
    try {
      oosOut.writeObject( strUserName ) ;
      oosOut.writeObject( strFileName ) ;
      oosOut.writeObject( strFilePath ) ;
      oosOut.writeObject( sw.toString() ) ;
      oosOut.flush() ;
      oosOut.close() ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 7 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "Buffer loaded." ) ;
    
    // Step 8 - set the length of the output buffer
    urlcConn.setRequestProperty( 
        "Content-Length", String.valueOf( baosByteStream.size() ) ) ;
    
    // Step 9 "is irrelevant when sending serialized data" (Hall & Brown)
    
    // Step 10 - actual data send
    try {
      baosByteStream.writeTo( urlcConn.getOutputStream() ) ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 8 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "Data sent." ) ;
   
    // begin steps to read response - straight ASCII
    
    // Step 11 - ASCII version (see p. 1073 Step 5)
    
    BufferedReader buffIn = null ;
    try {
      buffIn = new BufferedReader( new InputStreamReader( urlcConn.getInputStream() ) ) ;
    } catch ( FileNotFoundException fnfe ) {
      throw new UmlGuiException( 
          "\nFileNotFoundException at Step 11 in storeXMLDocument:" + 
          "\n   " + fnfe.toString() ) ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 11 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    // System.out.println( "buffIn = " + buffIn ) ;

    // Step 12 - ASCII version (see p. 1073 Step 6)
    
    String strReturned = "" ;
    String strLineRead = "" ;
    try {
      while ( ( strLineRead = buffIn.readLine() ) != null )
      {
        strReturned += strLineRead + "\n" ;
      }
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 12 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    
    // Step 13 - ASCII version (see p. 1073 Step 7)
    
    try {
     buffIn.close() ;
    } catch ( IOException ioe ) {
      throw new UmlGuiException( 
          "\nIOException at Step 13 in storeXMLDocument:" + 
          "\n   " + ioe.toString() ) ;
    }
    
    // return result
    return strReturned ;

  } // private static String storeXMLDocument(...)


  /** <p>This five-parameter <code>public static</code> method calls 
   *  <code>(Applet.getCodeBase()).getHost()</code> to get the name of the 
   *  host system on which the calling applet is running, which is required 
   *  to store the host on which to the XML document, and then calls the private 
   *  version of <code>storeXMLDocument</code> to do the actual save.</p>
   *  <p>Note that you do not usually want to call this version of this method
   *  because you will typically store your XML documents under the name in the
   *  directory from which the applet was run. This name can be extracted from 
   *  the applet's code base by the four-parameter version of this method.
   *  @param app          the calling applet
   *  @param doc          XML document to save
   *  @param strFileName  name under which to store the document
   *  @param strFilePath  subdirectory of the user area in which to store the 
   *                        document
   *  @param strUserName  name of user area in which to store the document
   *  @return a <code>String</code> indicating the full path to the XML document
   *      stored
   *  @throws UmlGuiException
   */
  public static String storeXMLDocument( Applet app, Document doc, 
        String strFileName, String strFilePath, String strUserName ) 
      throws UmlGuiException
  {
    return storeXMLDocument( (app.getCodeBase()).getHost(), doc, strFileName, 
        strFilePath, strUserName ) ;
  }


  /** This four-parameter <code>public static</code> method parses the user 
   *  name from <code>Applet.getCodeBase()</code> and then calls the five-parameter 
   *  version of this same method.
   *  @param app          the calling applet
   *  @param doc          XML document to save
   *  @param strFileName  name under which to store the document
   *  @param strFilePath  subdirectory of the user area in which to store the 
   *                        document
   *  @return a <code>String</code> indicating the full path to the XML document
   *      stored
   *  @throws UmlGuiException
   */
  public static String storeXMLDocument( Applet app, Document doc, 
        String strFileName, String strFilePath ) 
      throws UmlGuiException
  {
    // initialize user name to entire applet source string
    String strUserName = app.getCodeBase().toString() ;
    
    // find the name prefix, either "~", "%7e", or "%7E", and set the name 
    // starting position appropriately
    int posTilde = strUserName.indexOf( "~" ) ;
    if ( posTilde == -1 )
    {
      posTilde = strUserName.indexOf( "%7e" ) ;
      if ( posTilde == -1 )
      {
        posTilde = strUserName.indexOf( "%7E" ) ;
        if ( posTilde != -1 )
          posTilde += 3 ;
      }
      else
        posTilde += 3 ;
    }
    else
      posTilde += 1 ;
    
    // find the slash that marks the name ending position
    int posSlash = strUserName.indexOf( "/", posTilde ) ;

    // extract the user name or set to "undefined" if not found
    if ( posTilde != -1 && posSlash != -1 )
      strUserName = strUserName.substring( posTilde, posSlash ) ;
    else
      strUserName = "undefined" ;

    // call the five-parameter version of this method
    return storeXMLDocument( app, doc, strFileName, strFilePath, strUserName ) ;
  }
  
  
  /** This three-parameter <code>public static</code> method calls the 
   *  four-parameter version with the file path set to <code>null</code>.
   *  @param app          the calling applet
   *  @param doc          XML document to save
   *  @param strFileName  name under which to store the document
   *  @return a <code>String</code> indicating the full path to the XML document
   *      stored
   *  @throws UmlGuiException
   */
  public static String storeXMLDocument( Applet app, Document doc, 
        String strFileName ) 
      throws UmlGuiException
  {
    // call the four-parameter version of this method with the file path set to null
    return storeXMLDocument( app, doc, strFileName, null ) ;
  }
  
  
}
