package edu.uml.gui ;

/** UmlGuiPart1.java
 *  updated by JMH on September 27, 2002 at 07:29 PM
 */
import javax.xml.parsers.* ;    // for DocumentBuilderFactory and DocumentBuilder
import org.w3c.dom.* ;          // for Document and the Document Object Model
import org.xml.sax.* ;          // for SAXException
import java.io.* ;              // for IOException

/** This is Part 1 of the edu.uml.gui package.
 *  @author  Jesse M. Heines, <a href="mailto:heines@cs.uml.edu">heines@cs.uml.edu</a>
 *  @version 1.0, updated by JMH on October 05, 2002 at 11:15 AM
 */
public class UmlGuiPart1 {
  
  /** This <code>public static</code> method opens and XML file and returns the 
   *  <code>Document</code> object representing the XML document it contains.</p>
   *  <p>If the file cannot be opened or parsed, a <code>UmlGuiException</code> is thrown. 
   *  This is a checked exception derived from <code>IOException</code>, so this method 
   *  must be called from within a <code>try</code> block.
   *  @param strFilePath full path to the XML file to be opened
   *  @return <code>Document</code> object for that file
   *  @throws UmlGuiException
   */
  public static Document openXMLDocument( String strFilePath ) 
      throws UmlGuiException
  {
    // (1) get an instance of the DocumentBuilderFactory class
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance() ;
    
    // (2) use the instance of the DocumentBuilderFactory class to get an instance of the 
    // DocumentBuilder class, noting that the call to newDocumentBuilder() can throw a 
    // ParserConfigurationException
    DocumentBuilder db = null ;  // declare outside of try block 
    try {
      db = dbf.newDocumentBuilder() ;
    } catch ( ParserConfigurationException pce ) {
      String err = pce.toString() ;
      throw new UmlGuiException( 
          "\nParser Configuration Error (ParserConfigurationException) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    }

    // (3) use the instance of the DocumentBuilder class to open and parse the 
    // document, returning the Document object if successful or null if not
    try {
      return db.parse( strFilePath ) ;
    } catch ( SAXException saxe ) {
      String err = saxe.toString() ;
      throw new UmlGuiException( 
          "\nParsing Error (SAXException) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    } catch ( FileNotFoundException fnfe ) {
      String err = fnfe.toString() ;
      throw new UmlGuiException( 
          "\nFile Not Found Error ( FileNotFoundException ) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    } catch ( IOException ioe ) {
      String err = ioe.toString() ;
      throw new UmlGuiException( 
          "\nI/O Error ( IOException ) in openXMLDocument:" + 
          "\n   " + err.substring( 0, err.indexOf( ":" ) + 1 ) + 
          "\n   " + err.substring( err.indexOf( ":" ) + 2 ) ) ;
    }
  } // openXMLDocument(...)

} // class UmlGuiPart1
